//
//  FindMeViewController.m
//  FindMe
//
//  Created by Ken Anderson on 3/15/11.
//  Copyright 2011 University of Colorado, Boulder. All rights reserved.
//

#import "FindMeViewController.h"
#import "MapLocation.h"

@implementation FindMeViewController

@synthesize mapView, progressBar, progressLabel, button;

#pragma mark -

- (IBAction) findMe {
    
    CLLocationManager *lm = [[CLLocationManager alloc] init];
    [lm setDelegate:self];
    [lm setDesiredAccuracy:kCLLocationAccuracyBest];
    [lm startUpdatingLocation];
    
    [progressBar setHidden:NO];
    [progressBar setProgress:0.0];
    [progressLabel setText:@"Determining Current Location"];
    
    [button setHidden:YES];
}

- (void) openCallout: (id <MKAnnotation>) annotation {
    [progressBar setProgress:1.0];
    [progressLabel setText:@"Showing Annotation"];
    [mapView selectAnnotation:annotation animated:YES];
}

#pragma mark -

- (void) viewDidLoad {
    [mapView setMapType:MKMapTypeHybrid];
}

- (void) viewDidUnload {
    [self setMapView:nil];
    [self setProgressBar:nil];
    [self setProgressLabel:nil];
    [self setButton:nil];
}

- (void) dealloc {
    [mapView release];
    [progressBar release];
    [progressLabel release];
    [button release];
    [super dealloc];
}

- (BOOL) shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation {
    return YES;
}

#pragma mark -
#pragma mark CLLocationManagerDelegate Methods

- (void) locationManager:(CLLocationManager *)manager didUpdateToLocation:(CLLocation *)newLocation fromLocation:(CLLocation *)oldLocation {
    if ([[newLocation timestamp] timeIntervalSince1970] < [NSDate timeIntervalSinceReferenceDate] - 60) {
        return;
    }
    
    MKCoordinateRegion viewRegion = MKCoordinateRegionMakeWithDistance([newLocation coordinate], 2000, 2000);
    MKCoordinateRegion adjustedRegion = [mapView regionThatFits:viewRegion];
    [mapView setRegion:adjustedRegion animated:YES];
    
    [manager setDelegate:nil];
    [manager stopUpdatingLocation];
    [manager autorelease];
    
    [progressBar setProgress:0.25];
    [progressLabel setText:@"Reverse Geocoding Location"];
    
    MKReverseGeocoder *geocoder = [[MKReverseGeocoder alloc] initWithCoordinate:[newLocation coordinate]];
    [geocoder setDelegate:self];
    [geocoder start];

}

- (void) locationManager:(CLLocationManager *)manager didFailWithError:(NSError *)error {
    NSString *errorType = ([error code] == kCLErrorDenied) ? @"Access Denied" : @"Unknown Error";
    
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error Getting Location" message:errorType delegate:self cancelButtonTitle:@"Okay" otherButtonTitles:nil];
    [alert show];
    [alert release];

    [manager release];
}

#pragma mark -
#pragma mark Alert View Delegate Methods

- (void) alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex {
    [progressBar setHidden:YES];
    [progressLabel setText:@""];
}

#pragma mark -
#pragma mark Reverse Geocoder Delegate Methods

- (void) reverseGeocoder:(MKReverseGeocoder *)geocoder didFailWithError:(NSError *)error {
    UIAlertView *alert =
        [[UIAlertView alloc] initWithTitle:@"Reverse Geocoding Error"
                                   message:@"Geocoder did not recognize coordinates"
                                  delegate:self
                         cancelButtonTitle:@"Okay"
                         otherButtonTitles:nil];
    [alert show];
    [alert release];
    
    [geocoder setDelegate:nil];
    [geocoder autorelease];
    
}

- (void) reverseGeocoder:(MKReverseGeocoder *)geocoder didFindPlacemark:(MKPlacemark *)placemark {
    [progressBar setProgress:0.5];
    [progressLabel setText:@"Location Determined"];
    
    MapLocation *annotation = [[MapLocation alloc] init];
    [annotation setStreetAddress:[placemark thoroughfare]];
    [annotation setCity:[placemark locality]];
    [annotation setState:[placemark administrativeArea]];
    [annotation setZip:[placemark postalCode]];
    [annotation setCoordinate:[geocoder coordinate]];
    
    [mapView addAnnotation:annotation];
    
    [annotation release];

    [geocoder setDelegate:nil];
    [geocoder autorelease];

}

#pragma mark -
#pragma mark Map View Delegate Methods

- (MKAnnotationView *) mapView:(MKMapView *)theMapView viewForAnnotation:(id<MKAnnotation>)annotation {
    if ([annotation isKindOfClass:[MapLocation class]]) {
        MKPinAnnotationView *annotationView = (MKPinAnnotationView *)[theMapView dequeueReusableAnnotationViewWithIdentifier:@"Map Location"];
        if (annotationView == nil) {
            annotationView = [[MKPinAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:@"Map Location"];
        } else {
            [annotationView setAnnotation:annotation];
        }
        [annotationView setEnabled:YES];
        [annotationView setAnimatesDrop:YES];
        [annotationView setPinColor:MKPinAnnotationColorPurple];
        [annotationView setCanShowCallout:YES];
        [self performSelector:@selector(openCallout:) withObject:annotation afterDelay:0.5];
        
        [progressBar setProgress:0.75];
        [progressLabel setText:@"Creating Annotation"];
        
        return annotationView;
    }
    return nil;
}

- (void) mapViewDidFailLoadingMap:(MKMapView *)mapView withError:(NSError *)error {
    UIAlertView *alert =
    [[UIAlertView alloc] initWithTitle:@"Map Loading Error"
                               message:[error localizedDescription]
                              delegate:nil
                     cancelButtonTitle:@"Okay"
                     otherButtonTitles:nil];
    [alert show];
    [alert release];
}

@end
